from __future__ import absolute_import

import collections
import math
import numpy as np
import pp

import common.commonobjects as co


class DirtyImage(object):
    """Class to compute dirty image cube from uvspectra.
    """

    def __init__(self, previous_results, job_server):
        self.previous_results = previous_results
        self.job_server = job_server

        self.result = collections.OrderedDict()      

    def run(self):
        print 'DirtyImage.run'

        # get relevant fts info
        fts = self.previous_results['fts']
        fts_wnmin = fts['wnmin']

        # get primary beam info
        beamsgenerator = self.previous_results['beamsgenerator']
        npix = beamsgenerator['npix']
        spatial_axis = beamsgenerator['spatial axis [arcsec]']

        # get observation list
        uvspectra = self.previous_results['uvspectra']['uvspectra']
        wavenumber = uvspectra[0].wavenumber
        wavenumber = wavenumber[wavenumber>fts_wnmin]

        # dirty image cube
        dirtyimage = np.zeros([npix, npix, len(wavenumber)], np.float)
        dirtybeam = np.zeros([npix, npix, len(wavenumber)], np.float)

        for uvspectrum in uvspectra:

            b_x = uvspectrum.baseline_x
            b_y = uvspectrum.baseline_y
            spectrum = uvspectrum.spectrum
            wn_spectrum = uvspectrum.wavenumber

            for iwn,wn in enumerate(wavenumber):
                # must be a better way of doing this
                argx = b_x * (wn * 100.0) * spatial_axis / 206265.0
                argy = b_y * (wn * 100.0) * spatial_axis / 206265.0
                valx = np.exp(2j * math.pi * argx)
                valy = np.exp(2j * math.pi * argy)

                vis = spectrum[wn_spectrum==wn]

                # numpy arrays are [row,col] or here [y,x]
                for iy,y in enumerate(spatial_axis):
                    contribution = (vis * valx * valy[iy]) + \
                      np.conjugate(vis * valx * valy[iy])
                    dirtyimage[iy,:,iwn] += contribution.real
                    contribution = (valx * valy[iy]) + \
                      np.conjugate(valx * valy[iy])
                    dirtybeam[iy,:,iwn] += contribution.real
   
        # normalise
        for iwn,wn in enumerate(wavenumber):
            dirtyimage[:,:,iwn] /= np.sum(dirtybeam[:,:,iwn])
            dirtybeam[:,:,iwn] /= np.sum(dirtybeam[:,:,iwn])
#        dirtyimage /= 2.0 * float(len(uvspectra))
#        dirtybeam /= 2.0 * float(len(uvspectra))
        print 'sum image', np.sum(dirtyimage[:,:,0])
        print 'sum beam', np.sum(dirtybeam[:,:,0])

        self.result['dirtyimage'] = dirtyimage
        self.result['dirtybeam'] = dirtybeam
        self.result['spatial axis [arcsec]'] = spatial_axis
        self.result['spatial axis'] = spatial_axis
        self.result['wavenumber [cm-1]'] = wavenumber

        return self.result
                
    def __repr__(self):
        return '''
DirtyImage:
'''.format(
          num_uvspectra=len(self.uvspectra))

